from django.apps import AppConfig
from django.utils.module_loading import autodiscover_modules


class DjangoBridgeAppConfig(AppConfig):
    label = "django_bridge"
    name = "django_bridge"
    verbose_name = "Django Bridge"
    default_auto_field = "django.db.models.BigAutoField"

    def ready(self):
        autodiscover_modules("adapters")



from django.conf import settings
from django.contrib import messages
from django.http import JsonResponse
from django.utils.cache import patch_cache_control
from django.utils.html import conditional_escape
from django.utils.module_loading import import_string

from .adapters.registry import JSContext


def get_messages(request):
    return [
        {
            "level": (
                "error"
                if message.level == messages.ERROR
                else "warning"
                if message.level == messages.WARNING
                else "success"
            ),
            "html": conditional_escape(message.message),
        }
        for message in messages.get_messages(request)
    ]


class BaseResponse(JsonResponse):
    """
    Base class for all Django Bridge responses.
    """

    status = None

    def __init__(self, *args, http_status=None, **kwargs):
        js_context = JSContext()
        data = {
            "status": self.status,
        }
        data.update(js_context.pack(self.get_data(*args, **kwargs)))
        super().__init__(data, status=http_status)
        self["X-DjangoBridge-Status"] = self.status

        # Make sure that Django Bridge responses are never cached by browsers
        # We need to do this because Django Bridge responses are given on the same URLs that
        # users would otherwise get HTML responses on if they visited those URLs
        # directly.
        # If a Django Bridge response is cached, there's a chance that a user could see the
        # JSON document in their browser rather than a HTML page.
        # This behaviour only seems to occur (intermittently) on Firefox.
        patch_cache_control(self, no_store=True)

    def get_data(self):
        return {}


class Response(BaseResponse):
    """
    Instructs the client to render a view (React component) with the given context.
    """

    status = "render"

    def __init__(self, request, *args, overlay=False, title="", **kwargs):
        self.request = request
        self.overlay = overlay
        self.title = title
        super().__init__(*args, **kwargs)

    def get_data(self, view, props):
        return {
            "view": view,
            "overlay": self.overlay,
            "title": self.title,
            "props": props,
            "context": {
                name: import_string(provider)(self.request)
                for name, provider in settings.DJANGO_BRIDGE.get(
                    "CONTEXT_PROVIDERS", {}
                ).items()
            },
            "messages": get_messages(self.request),
        }


class ReloadResponse(BaseResponse):
    """
    Instructs the client to load the view the old-fashioned way.
    """

    status = "reload"


class RedirectResponse(BaseResponse):
    status = "redirect"

    def get_data(self, path):
        return {
            "path": path,
        }


class CloseOverlayResponse(BaseResponse):
    status = "close-overlay"

    def __init__(self, request, *args, **kwargs):
        self.request = request
        super().__init__(*args, **kwargs)

    def get_data(self):
        return {
            "messages": get_messages(self.request),
        }



from datetime import date, datetime, time

from django import forms

from .registry import Adapter, register, registry


class FieldWithName:
    def __init__(self, name, field):
        self.name = name
        self.field = field


class FieldAdapter(Adapter):
    def pack(self, field, context):
        if registry.find_adapter(field.field.field.widget.__class__) is None:
            # Field widget is not adaptable, render the widget on the server
            return (
                "forms.ServerRenderedField",
                [
                    field.name,
                    field.field.label,
                    field.field.field.required,
                    field.field.field.disabled,
                    field.field.help_text,
                    field.field.field.widget.render(field.name, field.field.value()),
                ],
            )

        # Render widget on the client
        value = field.field.value()

        if isinstance(value, (datetime, date, time)):
            value = value.isoformat()

        return (
            "forms.Field",
            [
                field.name,
                field.field.label,
                field.field.field.required,
                field.field.field.disabled,
                field.field.field.widget,
                field.field.help_text,
                value,
            ],
        )


register(FieldAdapter(), FieldWithName)


class FormAdapter(Adapter):
    js_constructor = "forms.Form"

    def js_args(self, form):
        return [
            [FieldWithName(name, form[name]) for name in form.fields.keys()],
            form.errors,
        ]


register(FormAdapter(), forms.BaseForm)



from datetime import datetime

from django import forms
from telepath import Adapter, AdapterRegistry, JSContextBase


class CustomJSContextBase(JSContextBase):
    # Remove default media as we import 'telepath-unpack' directly into the client app
    @property
    def base_media(self):
        return forms.Media()


class CustomAdapterRegistry(AdapterRegistry):
    js_context_base_class = CustomJSContextBase


registry = CustomAdapterRegistry()
JSContext = registry.js_context_class


def register(adapter, cls):
    registry.register(adapter, cls)


class DateTimeAdapter(Adapter):
    js_constructor = "Date"

    def js_args(self, date):
        return [date.isoformat()]


register(DateTimeAdapter(), datetime)



#!/usr/bin/env python

from os import path

from django_bridge import __version__
from setuptools import find_packages, setup

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "../README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="django-bridge",
    version=__version__,
    description="The simple way to build Django applications with modern React frontends",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Karl Hobley",
    author_email="karl@kaed.uk",
    url="https://django-bridge.org",
    project_urls={
        "Documentation": "https://django-bridge.org/docs/",
        "Source": "https://github.com/django-bridge/django-bridge/",
        "Tracker": "https://github.com/django-bridge/django-bridge/issues",
    },
    packages=find_packages(),
    include_package_data=True,
    license="BSD-3-Clause",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Framework :: Django",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 5.0",
    ],
    install_requires=["Django>=4.0,<6.0", "telepath>=0.3,<0.4"],
    zip_safe=False,
)



from django.views.generic.base import ContextMixin, View

from .response import Response


class DjangoBridgeMixin:
    """A mixin that can be used to render a view with a React component."""

    title = None
    view_name = None
    overlay = False
    response_class = Response

    def get_title(self):
        return self.title

    def render_to_response(self, props):
        """
        Return a response, using the `response_class` for this view, with a
        view rendered with the given props.

        Pass response_kwargs to the constructor of the response class.
        """
        return self.response_class(
            self.request,
            self.view_name,
            props,
            overlay=self.overlay,
            title=self.title,
        )


class DjangoBridgeView(DjangoBridgeMixin, ContextMixin, View):
    """
    A class-based view that can be used to render views with React.
    """

    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)
